@@ -17,9 +17,9 @@ import gzip
 from os import path
 from typing import List, Optional, Tuple
 
-import numpy as np
 import torch
 from torch.nn import Module
+from torch.utils.data import TensorDataset, random_split
 from tqdm import tqdm
 
 from neural_semigroups.constants import CAYLEY_DATABASE_PATH, CURRENT_DEVICE
@@ -63,28 +63,31 @@ class CayleyDatabase:
             if not path.exists(filename):
                 download_smallsemi_data(self.data_path)
             with gzip.open(filename, "rb") as file:
-                self.database = import_smallsemi_format(file.readlines())
-            self.labels = np.ones(len(self.database), dtype=np.int64)
+                self.database = import_smallsemi_format(file.readlines()).to(
+                    CURRENT_DEVICE
+                )
+            self.labels = torch.ones(
+                len(self.database), dtype=torch.int64, device=CURRENT_DEVICE
+            )
         else:
             check_filename(path.basename(database_filename))
-            npz_file = np.load(database_filename)
-            self.database = npz_file["database"]
-            self.labels = npz_file.get(
-                "labels", np.zeros(len(self.database), dtype=np.int64)
+            torch_zip_file = torch.load(database_filename)
+            self.database = torch_zip_file["database"]
+            self.labels = torch_zip_file.get(
+                "labels", torch.zeros(len(self.database), dtype=torch.int64)
             )
-            npz_file.close()
 
     def augment_by_equivalent_tables(self) -> None:
         """
         for every Cayley table in a previously loaded database adds all of its
         equivalent tables to the database
         """
-        database: List[np.ndarray] = []
+        database: List[torch.Tensor] = []
         for table in tqdm(
             self.database, desc="augmenting by equivalent tables"
         ):
             database += [get_equivalent_magmas(table)]
-        self.database = np.unique(np.concatenate(database, axis=0), axis=0)
+        self.database = torch.unique(torch.cat(database, dim=0), dim=0)
 
     def _check_input(self, cayley_table: List[List[int]]) -> bool:
         """
@@ -94,10 +97,11 @@ class CayleyDatabase:
         :returns: whether the input is correct
         """
         correct = True
-        table = np.array(cayley_table)
-        if table.shape != (self.cardinality, self.cardinality):
+        # pylint: disable=not-callable
+        table = torch.tensor(cayley_table)
+        if table.shape != torch.Size([self.cardinality, self.cardinality]):
             correct = False
-        elif table.dtype != int:
+        elif table.dtype != torch.long:
             correct = False
         elif table.max() >= self.cardinality or table.min() < -1:
             correct = False
@@ -105,7 +109,7 @@ class CayleyDatabase:
 
     def search_database(
         self, cayley_table: List[List[int]]
-    ) -> List[np.ndarray]:
+    ) -> List[torch.Tensor]:
         """
         get a list of possible completions of a partially filled Cayley table
         (unknown entries are filled by ``-1``)
@@ -119,18 +123,21 @@ class CayleyDatabase:
             )
         completions = list()
         if self.database is not None:
-            partial_table = np.array(cayley_table)
-            rows, cols = np.where(partial_table != -1)
+            # pylint: disable=not-callable
+            partial_table = torch.tensor(cayley_table)
+            rows, cols = torch.where(partial_table != -1)
             for table in tqdm(
                 self.database, desc="full scan over Cayley database"
             ):
-                if np.alltrue(table[rows, cols] == partial_table[rows, cols]):
+                if torch.allclose(
+                    table[rows, cols], partial_table[rows, cols]
+                ):
                     completions.append(table)
         return completions
 
     def fill_in_with_model(
         self, cayley_table: List[List[int]]
-    ) -> Tuple[np.ndarray, np.ndarray]:
+    ) -> Tuple[torch.Tensor, torch.Tensor]:
         """
         get a list of possible completions of a partially filled Cayley table
         (unknow entries are filled by ``-1``) using a machine learning model
@@ -146,33 +153,23 @@ class CayleyDatabase:
             raise ValueError(
                 f"invalid Cayley table of {self.cardinality} elements"
             )
-        table = np.array(cayley_table)
+        # pylint: disable=not-callable
+        table = torch.tensor(cayley_table)
         inv_cardinality = 1 / self.cardinality
-        cube = np.zeros(
+        cube = torch.zeros(
             [self.cardinality, self.cardinality, self.cardinality],
-            dtype=np.float32,
+            dtype=torch.float32,
+            device=CURRENT_DEVICE,
         )
-        rows, cols = np.where(table != -1)
+        rows, cols = torch.where(table != -1)
         cube[rows, cols, table[rows, cols]] = 1.0
-        rows, cols = np.where(table == -1)
+        rows, cols = torch.where(table == -1)
         cube[rows, cols, :] = inv_cardinality
-        prediction = (
-            self.model(
-                torch.from_numpy(
-                    cube.reshape(
-                        [
-                            -1,
-                            self.cardinality,
-                            self.cardinality,
-                            self.cardinality,
-                        ]
-                    )
-                ).to(CURRENT_DEVICE)
+        prediction = self.model(
+            cube.reshape(
+                [-1, self.cardinality, self.cardinality, self.cardinality]
             )
-            .cpu()
-            .detach()
-            .numpy()[0]
-        )
+        ).detach()[0]
         return (prediction.argmax(axis=-1), prediction)
 
     def load_model(self, filename: str) -> None:
@@ -192,25 +189,29 @@ class CayleyDatabase:
 
         :param cayley_db: a database of Cayley tables
         :param train_size: number of tables in a train set
-        :param train_size: number of tables in a validation set
+        :param validation_size: number of tables in a validation set
         :returns: a triple of distinct Cayley tables databases: ``(train, validation, test)``
         """
-        all_indices = np.arange(len(self.database))
-        np.random.shuffle(all_indices)
-        train_indices = all_indices[:train_size]
-        validation_indices = all_indices[
-            train_size : train_size + validation_size
+        splits = random_split(
+            TensorDataset(self.database, self.labels),
+            [
+                train_size,
+                validation_size,
+                self.database.shape[0] - train_size - validation_size,
+            ],
+        )
+        train = CayleyDatabase(self.cardinality)
+        train.database, train.labels = [
+            torch.stack(i) for i in zip(*splits[0])  # type: ignore
+        ]
+        validation = CayleyDatabase(self.cardinality)
+        validation.database, validation.labels = [
+            torch.stack(i) for i in zip(*splits[1])  # type: ignore
+        ]
+        test = CayleyDatabase(self.cardinality)
+        test.database, test.labels = [
+            torch.stack(i) for i in zip(*splits[2])  # type: ignore
         ]
-        test_indices = all_indices[train_size + validation_size :]
-        train = CayleyDatabase(self.cardinality, data_path=self.data_path)
-        train.database = self.database[train_indices]
-        train.labels = self.labels[train_indices]
-        validation = CayleyDatabase(self.cardinality, data_path=self.data_path)
-        validation.database = self.database[validation_indices]
-        validation.labels = self.labels[validation_indices]
-        test = CayleyDatabase(self.cardinality, data_path=self.data_path)
-        test.database = self.database[test_indices]
-        test.labels = self.labels[test_indices]
         return train, validation, test
 
     @property
@@ -229,16 +230,15 @@ class CayleyDatabase:
         """
         self._model = model
 
-    @property
-    def testing_report(self) -> np.ndarray:
+    def testing_report(self, max_level: int = -1) -> torch.Tensor:
         """
-        this functions:
+        this function:
 
         * takes 1000 random Cayley tables from the database
           (if there are less tables, it simply takes all of them)
-        * for each Cayley table generates ``cardinality ** 2 // 2`` puzzles
+        * for each Cayley table generates ``max_level`` puzzles
         * each puzzle is created from a table by omitting several cell values
-        * for each table the function omits 1, 2, and up to a half of all cells
+        * for each table the function omits 1, 2, and up to ``max_level`` of all cells
         * each puzzle is given to a pre-trained model of that database
         * if the model returns an associative table
           (not necessary the original one)
@@ -246,30 +246,32 @@ class CayleyDatabase:
         * in addition, all correctly filled cells are counted
           (despite leading to a full associative table)
 
+        :param max_level: up to how many cells to omit when creating a puzzle;
+            when not provided or explicitly set to ``-1`` it defaults to the total number of cells in a table
         :returns: statistics of solved puzzles splitted by the levels of difficulty
                   (number of cells omitted)
         """
         cardinality = self.cardinality
-        max_level = cardinality ** 2 // 2
-        totals = np.zeros((3, max_level), dtype=np.int32)
+        max_level = cardinality ** 2 if max_level == -1 else max_level
+        totals = torch.zeros((3, max_level))
         database_size = len(self.database)
-        test_indices = np.random.choice(
-            range(database_size), min(database_size, 1000), replace=False
-        )
+        test_indices = torch.randperm(database_size)[
+            : min(database_size, 1000)
+        ]
         for i in tqdm(test_indices, desc="generating and solving puzzles"):
             cayley_table = self.database[i]
             for level in range(1, max_level + 1):
                 rows, cols = zip(
                     *[
                         (point // cardinality, point % cardinality)
-                        for point in np.random.randint(
-                            0, cardinality ** 2, level
+                        for point in torch.randint(
+                            0, cardinality ** 2, [level]
                         )
                     ]
                 )
-                puzzle = cayley_table.copy()
+                puzzle = cayley_table.clone().detach()
                 puzzle[rows, cols] = -1
-                solution, _ = self.fill_in_with_model(puzzle)
+                solution, _ = self.fill_in_with_model(puzzle.tolist())
                 totals[0, level - 1] += 1
                 guessed_cells = sum(
                     solution[rows, cols] == cayley_table[rows, cols]
