@@ -16,7 +16,6 @@
 from collections import OrderedDict
 from typing import List, no_type_check
 
-import numpy as np
 import torch
 from torch import Tensor
 from torch.nn import BatchNorm1d, Linear, Module, ReLU, Sequential, Softmax2d
@@ -62,12 +61,12 @@ class MagmaDAE(Module):
             OrderedDict(reversed(decoder_layers.items()))
         ).to(CURRENT_DEVICE)
         self.encoder_layers = Sequential(encoder_layers).to(CURRENT_DEVICE)
-        self._nearly_zero = torch.from_numpy(
-            np.array([1e-6], dtype=np.float32)
-        ).to(CURRENT_DEVICE)
-        self._nearly_one = torch.from_numpy(
-            np.array([1 - (self.cardinality - 1) * 1e-6], dtype=np.float32)
-        ).to(CURRENT_DEVICE)
+        # pylint: disable=not-callable
+        self._nearly_zero = torch.tensor([1e-6], device=CURRENT_DEVICE)
+        # pylint: disable=not-callable
+        self._nearly_one = torch.tensor(
+            [1 - (self.cardinality - 1) * 1e-6], device=CURRENT_DEVICE
+        )
 
     def encode(self, corrupted_input: Tensor) -> Tensor:
         """
